@@ -34,7 +34,6 @@
      'filtered' .. event is not sent (but may be sent later)
      'delayed' ... event is sent with wall-clock delay
 
-
    1) P---| R		P normal, R normal
    2) R---| P		R normal, P normal
    3) P---R--| P	P normal, R filtered, delayed, P normal
@@ -61,7 +60,6 @@
    the second event). These cases are currently unhandled.
 */
 
-
 enum debounce_event {
 	DEBOUNCE_EVENT_PRESS = 50,
 	DEBOUNCE_EVENT_RELEASE,
@@ -522,7 +520,6 @@ fallback_debounce_handle_state(struct fallback_dispatch *dispatch,
 			flushed = false;
 		}
 
-
 		dispatch->debounce.button_code = changed[i];
 		debounce_handle_event(dispatch,
 				      is_down ?
@@ -568,13 +565,11 @@ fallback_init_debounce(struct fallback_dispatch *dispatch)
 	struct evdev_device *device = dispatch->device;
 	char timer_name[64];
 
-	if (device->model_flags &
-	    (EVDEV_MODEL_MS_NANO_TRANSCEIVER|EVDEV_MODEL_LOGITECH_K400)) {
+	if (device->model_flags & EVDEV_MODEL_BOUNCING_KEYS) {
 		dispatch->debounce.state = DEBOUNCE_STATE_DISABLED;
 		return;
 	}
 
-
 	dispatch->debounce.state = DEBOUNCE_STATE_IS_UP;
 
 	snprintf(timer_name,
