@@ -117,17 +117,12 @@ struct fallback_dispatch {
 		bool is_closed;
 		bool is_closed_client_state;
 
-		/* We allow up to 3 paired keyboards for the lid switch
+		/* We allow multiple paired keyboards for the lid switch
 		 * listener. Only one keyboard should exist, but that can
-		 * have more than one event node.
-		 *
-		 * Note: this is a sparse list, any element may have a
-		 * non-NULL device.
+		 * have more than one event node. And it's a list because
+		 * otherwise the test suite run fails too often.
 		 */
-		struct paired_keyboard {
-			struct evdev_device *device;
-			struct libinput_event_listener listener;
-		} paired_keyboard[3];
+		struct list paired_keyboard_list;
 	} lid;
 };
 
