@@ -41,7 +41,7 @@ static const char udev_rule[] =
 "ACTION==\"remove\", GOTO=\"switch_end\"\n"
 "KERNEL!=\"event*\", GOTO=\"switch_end\"\n"
 "\n"
-"ATTRS{name}==\"litest Lid Switch*\",\\\n"
+"ATTRS{name}==\"litest Lid Switch\",\\\n"
 "    ENV{ID_INPUT_SWITCH}=\"1\",\\\n"
 "    ENV{LIBINPUT_ATTR_LID_SWITCH_RELIABILITY}=\"reliable\"\n"
 "\n"
