@@ -26,13 +26,6 @@
 #include "litest.h"
 #include "litest-int.h"
 
-static void
-litest_protocol_a_touch_setup(void)
-{
-	struct litest_device *d = litest_create_device(LITEST_PROTOCOL_A_SCREEN);
-	litest_set_current_device(d);
-}
-
 static struct input_event down[] = {
 	{ .type = EV_ABS, .code = ABS_X, .value = LITEST_AUTO_ASSIGN },
 	{ .type = EV_ABS, .code = ABS_Y, .value = LITEST_AUTO_ASSIGN },
@@ -82,15 +75,13 @@ static int events[] = {
 	-1, -1,
 };
 
-struct litest_test_device litest_protocol_a_screen = {
+TEST_DEVICE("protocol-a",
 	.type = LITEST_PROTOCOL_A_SCREEN,
 	.features = LITEST_PROTOCOL_A,
-	.shortname = "protocol A",
-	.setup = litest_protocol_a_touch_setup,
 	.interface = &interface,
 
 	.name = "Protocol A touch screen",
 	.id = &input_id,
 	.events = events,
 	.absinfo = absinfo,
-};
+)
