@@ -1057,8 +1057,7 @@ START_TEST(gestures_3fg_buttonarea_scroll_btntool)
 }
 END_TEST
 
-void
-litest_setup_tests_gestures(void)
+TEST_COLLECTION(gestures)
 {
 	/* N, NE, ... */
 	struct range cardinals = { 0, 8 };
