@@ -372,6 +372,7 @@ main(int argc, char **argv)
 {
 	struct libinput *li;
 	struct libinput_event *ev;
+	bool grab = false;
 
 	/* This is kept for backwards-compatibility with the old
 	   libinput-list-devices */
@@ -388,7 +389,7 @@ main(int argc, char **argv)
 		}
 	}
 
-	li = tools_open_backend(BACKEND_UDEV, "seat0", false, false);
+	li = tools_open_backend(BACKEND_UDEV, "seat0", false, &grab);
 	if (!li)
 		return 1;
 
