@@ -193,16 +193,16 @@ usage(void)
 	printf("Usage: %s [options] [dx1] [dx2] [...] > gnuplot.data\n", program_invocation_short_name);
 	printf("\n"
 	       "Options:\n"
-	       "--mode=<motion|accel|delta|sequence> \n"
-	       "	motion   ... print motion to accelerated motion (default)\n"
+	       "--mode=<accel|motion|delta|sequence> \n"
+	       "	accel    ... print accel factor (default)\n"
+	       "	motion   ... print motion to accelerated motion\n"
 	       "	delta    ... print delta to accelerated delta\n"
-	       "	accel    ... print accel factor\n"
 	       "	sequence ... print motion for custom delta sequence\n"
 	       "--maxdx=<double>  ... in motion mode only. Stop increasing dx at maxdx\n"
 	       "--steps=<double>  ... in motion and delta modes only. Increase dx by step each round\n"
 	       "--speed=<double>  ... accel speed [-1, 1], default 0\n"
 	       "--dpi=<int>	... device resolution in DPI (default: 1000)\n"
-	       "--trackpoint_range=<int> ... range of the trackpoint deltas (default: 30)\n"
+	       "--trackpoint-range=<int> ... range of the trackpoint deltas (default: 20)\n"
 	       "--filter=<linear|low-dpi|touchpad|x230|trackpoint> \n"
 	       "	linear	  ... the default motion filter\n"
 	       "	low-dpi	  ... low-dpi filter, use --dpi with this argument\n"
@@ -222,6 +222,13 @@ usage(void)
 	       "Output best viewed with gnuplot. See output for gnuplot commands\n");
 }
 
+enum mode {
+	ACCEL,
+	MOTION,
+	DELTA,
+	SEQUENCE,
+};
+
 int
 main(int argc, char **argv)
 {
@@ -229,10 +236,7 @@ main(int argc, char **argv)
 	double step = 0.1,
 	       max_dx = 10;
 	int nevents = 0;
-	bool print_accel = false,
-	     print_motion = true,
-	     print_delta = false,
-	     print_sequence = false;
+	enum mode mode = ACCEL;
 	double custom_deltas[1024];
 	double speed = 0.0;
 	int dpi = 1000;
@@ -280,13 +284,13 @@ main(int argc, char **argv)
 			break;
 		case OPT_MODE:
 			if (streq(optarg, "accel"))
-				print_accel = true;
+				mode = ACCEL;
 			else if (streq(optarg, "motion"))
-				print_motion = true;
+				mode = MOTION;
 			else if (streq(optarg, "delta"))
-				print_delta = true;
+				mode = DELTA;
 			else if (streq(optarg, "sequence"))
-				print_sequence = true;
+				mode = SEQUENCE;
 			else {
 				usage();
 				return 1;
@@ -357,8 +361,7 @@ main(int argc, char **argv)
 
 	if (!isatty(STDIN_FILENO)) {
 		char buf[12];
-		print_sequence = true;
-		print_motion = false;
+		mode = SEQUENCE;
 		nevents = 0;
 		memset(custom_deltas, 0, sizeof(custom_deltas));
 
@@ -366,25 +369,33 @@ main(int argc, char **argv)
 			custom_deltas[nevents++] = strtod(buf, NULL);
 		}
 	} else if (optind < argc) {
-		print_sequence = true;
-		print_motion = false;
+		mode = SEQUENCE;
 		nevents = 0;
 		memset(custom_deltas, 0, sizeof(custom_deltas));
 		while (optind < argc)
 			custom_deltas[nevents++] = strtod(argv[optind++], NULL);
+	} else if (mode == SEQUENCE) {
+		usage();
+		return 1;
 	}
 
-	if (print_accel) {
+	switch (mode) {
+	case ACCEL:
 		if (!profile) /* trackpoint */
 			print_accel_func_trackpoint(filter, tp_range_max);
 		else
 			print_accel_func(filter, profile, dpi);
-	} else if (print_delta)
+		break;
+	case DELTA:
 		print_ptraccel_deltas(filter, step);
-	else if (print_motion)
+		break;
+	case MOTION:
 		print_ptraccel_movement(filter, nevents, max_dx, step);
-	else if (print_sequence)
+		break;
+	case SEQUENCE:
 		print_ptraccel_sequence(filter, nevents, custom_deltas);
+		break;
+	}
 
 	filter_destroy(filter);
 
