@@ -232,7 +232,7 @@ open_restricted(const char *path, int flags, void *user_data)
 	if (fd < 0)
 		fprintf(stderr, "Failed to open %s (%s)\n",
 			path, strerror(errno));
-	else if (*grab && ioctl(fd, EVIOCGRAB, (void*)1) == -1)
+	else if (grab && *grab && ioctl(fd, EVIOCGRAB, (void*)1) == -1)
 		fprintf(stderr, "Grab requested, but failed for %s (%s)\n",
 			path, strerror(errno));
 
@@ -251,7 +251,7 @@ static const struct libinput_interface interface = {
 };
 
 static struct libinput *
-tools_open_udev(const char *seat, bool verbose, bool grab)
+tools_open_udev(const char *seat, bool verbose, bool *grab)
 {
 	struct libinput *li;
 	struct udev *udev = udev_new();
@@ -261,16 +261,15 @@ tools_open_udev(const char *seat, bool verbose, bool grab)
 		return NULL;
 	}
 
-	li = libinput_udev_create_context(&interface, &grab, udev);
+	li = libinput_udev_create_context(&interface, grab, udev);
 	if (!li) {
 		fprintf(stderr, "Failed to initialize context from udev\n");
 		goto out;
 	}
 
-	if (verbose) {
-		libinput_log_set_handler(li, log_handler);
+	libinput_log_set_handler(li, log_handler);
+	if (verbose)
 		libinput_log_set_priority(li, LIBINPUT_LOG_PRIORITY_DEBUG);
-	}
 
 	if (libinput_udev_assign_seat(li, seat)) {
 		fprintf(stderr, "Failed to set seat\n");
@@ -285,12 +284,12 @@ out:
 }
 
 static struct libinput *
-tools_open_device(const char *path, bool verbose, bool grab)
+tools_open_device(const char *path, bool verbose, bool *grab)
 {
 	struct libinput_device *device;
 	struct libinput *li;
 
-	li = libinput_path_create_context(&interface, &grab);
+	li = libinput_path_create_context(&interface, grab);
 	if (!li) {
 		fprintf(stderr, "Failed to initialize context from %s\n", path);
 		return NULL;
@@ -315,7 +314,7 @@ struct libinput *
 tools_open_backend(enum tools_backend which,
 		   const char *seat_or_device,
 		   bool verbose,
-		   bool grab)
+		   bool *grab)
 {
 	struct libinput *li;
 
